#!/usr/bin/env python3
"""
Module MAIN
Sub-Package TEST.IO of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This is the test-running script for the PLIB3.IO test suite.
"""


if __name__ == '__main__':
    from plib.test.support import run_tests
    
    run_tests(__name__)



#!/usr/bin/env python3
"""
Module AsyncCommunicator
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous AsyncCommunicator class.
"""


class AsyncCommunicator(object):
    """Communicator mixin class for async I/O.
    
    Adds checking of the ``done`` flag to the ``do_loop`` method,
    so we break out of the loop if the flag is set. Note that
    calling the ``do_loop`` method implies that this object is
    the "master" async I/O object; it should *not* be called,
    for example, in request handlers.
    """
    
    def do_loop(self, callback=None):
        """Assume we're controlling the loop, so check ``done`` to exit.
        """
        if callback is not None:
            def f():
                if (callback() is False) or self.done:
                    return False
        else:
            def f():
                if self.done:
                    return False
        super(AsyncCommunicator, self).do_loop(f)
    
    def run_loop(self, callback=None):
        self.do_loop(callback)



#!/usr/bin/env python3
"""
TEST.IO.TESTLIB_ASYNC.PY -- utility module for PLIB3 I/O tests
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common code for the tests of the async
I/O modules in PLIB3.STDLIB.
"""

from plib.io.mixins import EchoRequestMixin

import plib.io.async  # FIXME: why is this needed to make things work?

from plib.io.async import (
    SocketClient, BaseRequestHandler, SocketServer,
    SocketClientWithShutdown, BaseRequestHandlerWithShutdown,
    SocketClientWithTerminator, BaseRequestHandlerWithTerminator,
    SocketClientWithReadWrite, BaseRequestHandlerWithReadWrite
)

from .testlib import IOClientMixin, IOServerMixin


class AsyncClient(IOClientMixin, SocketClient):
    pass


class AsyncHandler(EchoRequestMixin, BaseRequestHandler):
    pass


class AsyncServer(IOServerMixin, SocketServer):
    pass


class AsyncTestMixin(object):
    
    client_class = AsyncClient
    handler_class = AsyncHandler
    server_class = AsyncServer


class AsyncShutdownClient(IOClientMixin,
                          SocketClientWithShutdown):
    pass


class AsyncShutdownHandler(EchoRequestMixin,
                           BaseRequestHandlerWithShutdown):
    pass


class AsyncShutdownTestMixin(object):
    
    client_class = AsyncShutdownClient
    handler_class = AsyncShutdownHandler
    server_class = AsyncServer


class AsyncTerminatorClient(IOClientMixin,
                            SocketClientWithTerminator):
    pass


class AsyncTerminatorHandler(EchoRequestMixin,
                             BaseRequestHandlerWithTerminator):
    pass


class AsyncTerminatorTestMixin(object):
    
    client_class = AsyncTerminatorClient
    handler_class = AsyncTerminatorHandler
    server_class = AsyncServer


class AsyncReadWriteClient(IOClientMixin,
                           SocketClientWithReadWrite):
    pass


class AsyncReadWriteHandler(EchoRequestMixin,
                            BaseRequestHandlerWithReadWrite):
    pass


class AsyncReadWriteTestMixin(object):
    
    client_class = AsyncReadWriteClient
    handler_class = AsyncReadWriteHandler
    server_class = AsyncServer



#!/usr/bin/env python3
"""
Module ClientCommunicator
Sub-Package IO.COMM of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ClientCommunicator class.
"""

from plib.io.base import BaseCommunicator


class ClientCommunicator(BaseCommunicator):
    """Communicator specialized for client-side I/O.
    
    Assumes a "client" data pattern: writes ``write_data``,
    then reads back ``read_data``, processes the data, then
    clears data and checks if done. Call ``client_communicate``
    to run the communication loop. Override ``process_data``
    to do something with the received data. Expects a class
    earlier in the MRO to implement the ``start``, ``handle_read``,
    ``handle_write``, ``read_complete``, ``write_complete``,
    ``clear_read``, and ``clear_write`` methods (the intent
    is that this will be ``BaseData`` or a class derived
    from it).
    """
    
    waiting = False
    
    def writable(self):
        return not (self.waiting or self.write_complete() or self.done)
    
    def handle_write(self):
        super(ClientCommunicator, self).handle_write()
        if self.write_complete():
            self.clear_write()
    
    def readable(self):
        return not (self.writable() or self.read_complete() or self.done)
    
    def handle_read(self):
        super(ClientCommunicator, self).handle_read()
        if self.read_complete():
            self.process_data()
            self.clear_read()
            self.check_done()
    
    def query_done(self):
        """Clients should exit the communication loop if no data pending.
        
        If ``keep_alive`` is true, the channel will remain open, but
        no more data will be exchanged until ``client_communicate`` is
        called again.
        """
        
        return (not self.data_pending) or \
            super(ClientCommunicator, self).query_done()
    
    def wait_for(self, obj, attrname):
        """Force reading only until attrname of obj is True.
        
        Main intended use case is to call this method before starting the
        communication loop for cases where an initial "greeting" is
        expected from the server before we can start sending data.
        """
        
        self.waiting = True
        self.run_wait_loop(obj, attrname)
        self.waiting = False
    
    def client_communicate(self, data, client_id=None, callback=None):
        """Push data for writing and run client-side communication loop.
        
        Core method to implement client I/O functionality:
        connects to server, writes data, reads back the reply,
        processes it, then checks to see if it is done.
        
        Note that if the ``query_done`` method returns false
        (it will be called from ``check_done`` on each iteration),
        it should either call the ``start`` method to supply
        more data to be written, or ensure that ``write_complete``
        is ``True``; otherwise the client may block forever
        waiting for data.
        
        Also note that if ``client_id is None``, we assume that
        the client is already connected to the server and don't
        try to connect again.
        """
        
        self.start(data)
        if client_id is not None:
            self.setup_client(client_id)
        self.run_loop(callback)
    
    # Methods which must be implemented by derived classes
    
    def setup_client(self, client_id):
        """Prepare client for communication.
        """
        raise NotImplementedError
    
    def run_wait_loop(self, obj, attrname):
        """Run read-only communication loop until obj.attrname is true.
        
        Will also exit loop if self.query_done() is true.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Module RequestBase
Sub-Package IO.BLOCKING of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O RequestBase class.
"""

import sys

from plib.io.socket import BaseRequest
from plib.io.blocking import SocketBase


class RequestBase(BaseRequest, SocketBase):
    """Base class for blocking socket request handler.
    """
    
    def __init__(self, request, client_addr, server):
        BaseRequest.__init__(self, request, client_addr, server)
        SocketBase.__init__(self, request)
        
        # If we're being called at all, we must be connected
        self.handle_connect()
        
        # A blocking request handler runs its own comm loop
        try:
            self.run_loop()
        finally:
            self.close()



#!/usr/bin/env python3
"""
Module SERIAL -- Serial Port Handling
Sub-Package IO.SERIAL of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains a thin wrapper class around the
``Serial`` class from the pyserial package. The wrapper just
adds some options for using the class that work around
potential quirks in various types of serial devices. Note
that the wrapper class is only implemented for POSIX
systems; on other systems the only class added by this
module is the data overlay class.
"""

import os
import errno

from plib.io.base import BaseData, BaseIO

if os.name != 'posix':
    from serial import Serial

else:
    import fcntl
    import serial  # provided by pyserial package
    from serial import SerialException, EIGHTBITS, PARITY_NONE, STOPBITS_ONE
    
    
    class Serial(serial.Serial):
        """
        Thin wrapper class around ``serial.Serial``. Key changes:
        
        - Adds option to go back into blocking mode after the
          port is opened (the open needs to be non-blocking to
          avoid indefinite waits on devices).
        - Eliminates the select call from the read and write
          methods; it isn't needed since this class is intended
          to be mixed in with the ``AsyncBase`` async I/O class,
          which handles the select functionality (see the
          ``async.SerialDispatcher`` module for the implementation
          of this).
        - The ``read`` method may return fewer bytes than the size
          parameter instead of looping until that many bytes are
          read, and it raises an exception on any error except
          ``EAGAIN`` (for which it returns an empty string).
        - The ``write`` method returns the number of bytes written
          instead of looping until it's all written.
        - Because of the above changes to ``read`` and ``write``,
          the timeout parameters have no effect; instead, use the
          ``blocking_mode`` flag to control read and write behavior.
        
        The changes to the read and write methods make this a more
        low-level class; the higher-level classes overlaid on this
        one in the ``async`` and ``blocking`` sub-packages then take
        care of managing whether all desired bytes have been read or
        written, using the common API for all I/O types.
        """
        
        def __init__(self, port=None, baudrate=9600,
                     bytesize=EIGHTBITS, parity=PARITY_NONE, stopbits=STOPBITS_ONE,
                     xonxoff=0, rtscts=0, dsrdtr=None, blocking_mode=False):
            
            self.blocking_mode = blocking_mode
            # Timeouts are None since they're not used here
            serial.Serial.__init__(self, port, baudrate, bytesize,
                                   parity, stopbits,
                                   None, xonxoff, rtscts, None, dsrdtr)
        
        def open(self):
            if self._port is None:
                raise SerialException(
                    "Port must be configured before it can be used.")
            self.fd = None
            try:
                self.fd = os.open(self.portstr,
                                  os.O_RDWR | os.O_NOCTTY | os.O_NONBLOCK)
            except Exception as msg:
                self.fd = None
                raise SerialException("could not open port {:d}: {}".format(
                                      self._port, msg))
            
            if self.blocking_mode:
                # Make sure we don't clobber any other flags
                flags = fcntl.fcntl(self.fd, fcntl.F_GETFL)
                flags &= ~os.O_NONBLOCK
                fcntl.fcntl(self.fd, fcntl.F_SETFL, flags)
            
            try:
                self._reconfigurePort()
            except:
                os.close(self.fd)
                self.fd = None
            else:
                self._isOpen = True
        
        def read(self, size=1):
            if self.fd is None:
                raise portNotOpenError
            try:
                data = os.read(self.fd, size)
                return data
            except OSError as err:
                if err.errno != errno.EAGAIN:
                    raise
                return ""
        
        def write(self, data):
            if self.fd is None:
                raise portNotOpenError
            if not isinstance(data, str):
                raise TypeError('expected str, got {}'.format(type(data)))
            try:
                n = os.write(self.fd, data)
                return n
            except OSError as err:
                if err.errno != errno.EAGAIN:
                    raise
                return 0


# Now the base I/O management class, using the above to talk to
# the serial port

class SerialIOBase(BaseIO):
    """
    Base class for serial I/O -- no frills, just reads
    and writes the serial device file. The ``blocking_mode``
    flag controls whether the underlying serial object will
    use blocking or non-blocking I/O.
    """
    
    blocking_mode = False
    
    closed = True
    
    def __init__(self, devid=None):
        if devid is not None:
            self.create_port(devid)
        else:
            self.port = None
    
    def create_port(self, devid):
        self.set_port(Serial(devid, blocking_mode=self.blocking_mode))
    
    def set_port(self, port):
        self.port = port
        self.closed = False
    
    # BaseIO method implementations
    
    def dataread(self, bufsize):
        return self.port.read(self.bufsize)
    
    def datawrite(self, data):
        return self.port.write(self.write_data)
    
    def close(self):
        if self.port is not None:
            self.port.close()
            self.port = None
            self.closed = True


class SerialData(BaseData):
    """Basic data handling for serial I/O.
    """
    
    bufsize = 1



#!/usr/bin/env python3
"""
TEST.IO.TEST_MULTIREQUEST.PY -- test script for sub-package IO of package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests to ensure that the I/O servers in the
PLIB3.IO sub-package can handle multiple requests, both in sequence
and concurrent.
"""

import sys
import time
import unittest
from functools import partial

from .testlib import ConcurrentRequestTest, MultiRequestTest
from .testlib_async import AsyncClient, AsyncHandler, AsyncServer
from .testlib_blocking import (
    BlockingClient, BlockingHandler,
    ThreadingServer
)

if sys.platform != 'win32':
    # The forking server currently does not work on Windows
    from .testlib_blocking import ForkingServer


# For blocking I/O, implement delay by having each request
# handler call sleep before processing data

class SleepingHandler(BlockingHandler):
    
    def process_data(self):
        time.sleep(0.5)
        super(SleepingHandler, self).process_data()


if sys.platform != 'win32':
    
    class ForkingTestMixin(object):
        
        client_class = BlockingClient
        server_class = ForkingServer
    
    
    class ForkingConcurrentRequestTest(ForkingTestMixin,
                                       ConcurrentRequestTest,
                                       unittest.TestCase):
        
        handler_class = SleepingHandler
    
    
    class ForkingConcurrentRequestTestWithGoCode(ForkingTestMixin,
                                                 ConcurrentRequestTest,
                                                 unittest.TestCase):
        
        handler_class = BlockingHandler
        use_gocode = True
    
    
    class ForkingMultiRequestTest(ForkingTestMixin,
                                  MultiRequestTest,
                                  unittest.TestCase):
        
        handler_class = BlockingHandler


class ThreadingTestMixin(object):
    
    client_class = BlockingClient
    server_class = ThreadingServer


class ThreadingConcurrentRequestTest(ThreadingTestMixin,
                                     ConcurrentRequestTest,
                                     unittest.TestCase):
    
    handler_class = SleepingHandler


class ThreadingConcurrentRequestTestWithGoCode(ThreadingTestMixin,
                                               ConcurrentRequestTest,
                                               unittest.TestCase):
    
    handler_class = BlockingHandler
    use_gocode = True


class ThreadingMultiRequestTest(ThreadingTestMixin,
                                MultiRequestTest,
                                unittest.TestCase):
    
    handler_class = BlockingHandler


# For async I/O, since the handlers can't block the overall
# communication loop, implement delay by having the master
# loop keep track of paused handlers

paused_handlers = {}


class PausingHandler(AsyncHandler):
    
    paused = b""
    
    def process_data(self):
        global paused_handlers
        self.paused = self.read_data
        paused_handlers[self._fileno] = (time.time(), self)
    
    def unpause(self):
        global paused_handlers
        del paused_handlers[self._fileno]
        self.start(self.paused)
        self.paused = b""


def pause_callback(self, callback):
    result = callback()
    if (result is not False) and paused_handlers:
        self.check_paused_handlers()
    return result


def pause_no_callback(self):
    if paused_handlers:
        self.check_paused_handlers()


class PausingServer(AsyncServer):
    
    poll_timeout = 0.5
    
    def check_paused_handlers(self):
        unpaused = []
        for key, value in paused_handlers.items():
            t, handler = value
            if time.time() > (t + 0.5):
                unpaused.append(handler)
        for handler in unpaused:
            handler.unpause()
    
    def do_loop(self, callback=None):
        if callback is not None:
            f = partial(pause_callback, self, callback)
        else:
            f = partial(pause_no_callback, self)
        super(PausingServer, self).do_loop(f)


class NonBlockingConcurrentRequestTest(ConcurrentRequestTest,
                                       unittest.TestCase):
    
    client_class = AsyncClient
    handler_class = PausingHandler
    server_class = PausingServer


class NonBlockingConcurrentRequestTestWithGoCode(ConcurrentRequestTest,
                                                 unittest.TestCase):
    
    client_class = AsyncClient
    handler_class = AsyncHandler
    server_class = AsyncServer
    use_gocode = True


class NonBlockingMultiRequestTest(MultiRequestTest,
                                  unittest.TestCase):
    
    client_class = AsyncClient
    handler_class = AsyncHandler
    server_class = AsyncServer


if __name__ == '__main__':
    if sys.platform == 'win32':
        print("Forking server disabled, skipping forking multi-request tests.")
    unittest.main()



#!/usr/bin/env python3
"""
Module SocketBase
Sub-Package IO.BLOCKING of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O SocketBase class.
"""

from plib.io.socket import SocketData, SocketIOBase


class SocketBase(SocketData, SocketIOBase):
    """Base blocking socket I/O class with data handling.
    """
    
    def handle_connect(self):
        self.on_connect()
    
    def on_connect(self):
        """Placeholder for derived classes.
        """
        pass
    
    def readable(self):
        """We can always read from a blocking socket.
        """
        return True
    
    def writable(self):
        """We can always write to a blocking socket.
        """
        return True
    
    def close(self):
        if not self.closed:
            self.handle_close()
        super(SocketBase, self).close()
    
    def handle_close(self):
        self.on_close()
    
    def on_close(self):
        """Placeholder for derived classes.
        """
        pass



#!/usr/bin/env python3
"""
Module BaseServer
Sub-Package IO.SOCKET of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the socket BaseServer class.
"""

import socket


class BaseServer(object):
    """Base socket server class
    
    Sets up socket to listen on specified address.
    """
    
    address_family = socket.AF_INET
    allow_reuse_address = False
    request_queue_size = 5
    socket_type = socket.SOCK_STREAM
    
    def __init__(self, server_addr, handler_class):
        self.handler = handler_class
        
        self.server_start()  # for initialization before socket is created
        
        self.create_socket(self.address_family, self.socket_type)
        if self.allow_reuse_address:
            self.set_reuse_addr()
        self.bind(server_addr)
        self.listen(self.request_queue_size)
    
    def serve_forever(self, callback=None):
        """Handle requests until doomsday.
        
        Runs the accept/request handling loop, and ensures that
        the server is closed on loop exit.
        
        A ``callback`` parameter is provided for cases where the
        ``keep_running`` hook is not suitable. The callback must be
        a zero-argument callable; it should return ``False`` (not
        just a false value, but the specific object) if the loop
        should exit. Requiring the specific object ``False`` is so
        that callbacks can, if desired, be procedures that do not
        return a value (meaning the return defaults to ``None``).
        """
        
        try:
            self.server_loop(callback)
        finally:
            self.close()
    
    # Methods which may be overridden by derived classes
    
    def server_start(self):
        """Placeholder for derived classes.
        """
        pass
    
    def keep_running(self):
        """Check whether server should break out of the serve_forever loop.
        
        The ``server_loop`` method should check this method every time
        it begins a new iteration of the loop. This method is factored
        out to provide a convenient hook within the loop. Note that if
        this method is not overridden, the loop should never exit unless
        an exception occurs.
        """
        return True
    
    def server_close(self):
        """Placeholder for derived classes.
        """
        pass
    
    # Methods which must be implemented by derived classes
    
    def server_loop(self, callback=None):
        """Run the accept/request handling loop.
        
        This method must be implemented in derived classes to run
        the standard socket server loop: wait for an incoming
        request, accept it, and hand it off to a request handler.
        It must also support the semantics described above for
        triggering a loop exit (the ``keep_running`` method returning
        a false value, or the ``callback`` callable, if present,
        returning the specific object ``False``).
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Sub-Package IO.BLOCKING of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package includes classes for handling blocking
I/O, including both serial and socket I/O channels. See
the docstring for the parent package, ``plib.io``,
for information on how this package fits into the overall
I/O package structure.
"""

from plib.io.utils import IOModuleProxy

IOModuleProxy(__name__).init_proxy(__name__, __path__, globals(), locals())

# Now clean up our namespace
del IOModuleProxy



#!/usr/bin/env python3
"""
Module RequestBase
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous RequestBase class.
"""

from plib.io.socket import BaseRequest
from plib.io.async import SocketBase


class RequestBase(BaseRequest, SocketBase):
    """Base class for async request handler.
    """
    
    def __init__(self, request, client_addr, server):
        BaseRequest.__init__(self, request, client_addr, server)
        SocketBase.__init__(self, request)
        
        # If we're being called at all, we must be connected
        self.handle_connect()



#!/usr/bin/env python3
"""
Module BaseRequest
Sub-Package IO.SOCKET of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the socket I/O BaseRequest class.
"""


class BaseRequest(object):
    """Base socket I/O class set up to serve as a request handler.
    """
    
    def __init__(self, request, client_addr, server):
        self.request = request
        self.client_address = client_addr
        self.server = server



#!/usr/bin/env python3
"""
CHAT_CLIENT.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Python implementation of a "chat" client, which reads
input from stdin and sends it to a chat server, and gets
data sent from the server and displays it. The chat
client and server both demonstrate simple but illustrative
use cases for the asynchronous "persistent" I/O classes
in ``plib.io.async``. Key aspects of the client:

- The "asynchronous" part: this client multiplexes reading
  the user's typed messages on standard input with socket
  I/O, and both types of input can arrive at any time. This
  type of I/O is virtually impossible to handle with blocking
  reads--certainly not while keeping the user interface
  responsive. In principle threads could be used to keep the
  UI responsive while socket I/O was taking place, and vice
  versa, but then you would have to have a mechanism for
  passing the data read from stdin in the UI thread to the
  socket I/O thread for writing to the socket, which brings
  in all the well-known issues with mutexes, etc. This client
  neatly avoids all of that.

- The "persistent" part: the client may have to write user
  messages to the socket interleaved in arbitrary order with
  reading chat messages coming from the server. A normal
  "client" implementation has to make assumptions about when
  and in what order the two types of I/O, reading and writing,
  will occur. This client does not; it simply handles whatever
  happens, in the order in which it happens.
"""

import sys
import os

from plib.io.async import (
    AsyncBase, PersistentSocketWithTerminator)


class AsyncStdin(AsyncBase):
    
    def __init__(self, chat):
        AsyncBase.__init__(self)
        self.chat = chat
        self.i = sys.stdin
        self.set_fileobj(sys.stdin)
    
    def writable(self):
        # Can't write to standard input!
        return False
    
    def readable(self):
        # This disables reading user input until the greeting is received
        return self.chat.greeting_received
    
    def handle_read(self):
        self.chat.stdin_data(self.i.readline())


class ChatClient(PersistentSocketWithTerminator):
    """Chat client for testing.
    
    Waits for greeting message when first connecting. Prints all
    received data to stdout. Exits on Ctrl-C or EOF (Ctrl-D),
    or by typing "QUIT".
    """
    
    terminator = os.linesep.encode()
    
    server_addr = ("localhost", 9999)
    QUIT = "QUIT"
    
    greeting_received = False
    
    def on_connect(self):
        print("Got connection!")
        self.async_stdin = AsyncStdin(self)
    
    def writable(self):
        # This disables sending data until the greeting is received
        return self.greeting_received and super(ChatClient, self).writable()
    
    def process_data(self):
        print(self.read_data.decode())
        if not self.greeting_received:
            self.greeting_received = True
    
    def stdin_data(self, data):
        if data == self.QUIT:
            raise EOFError("QUIT received")
        self.start(data.encode())
    
    def run(self):
        # Run interactive session
        try:
            self.do_connect(self.server_addr)
            self.do_loop()
        except (KeyboardInterrupt, EOFError):
            self.close()
    
    def on_close(self):
        self.async_stdin.close()
        print("Closing connection!")


if __name__ == '__main__':
    ChatClient().run()



#!/usr/bin/env python3
"""
Module ClientMixin
Sub-Package IO.BLOCKING of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O ClientMixin class.
"""

from plib.io.comm import ClientCommunicator


class ClientMixin(ClientCommunicator):
    """Mixin class for blocking client-side I/O channel.
    """
    
    def run_loop(self, callback=None):
        try:
            while 1:
                while self.writable():
                    self.handle_write()
                while self.readable():
                    self.handle_read()
                if callback is not None:
                    c = callback()
                else:
                    c = None
                if self.done or (c is False):
                    break
        
        except:
            # We can only call close here on an exception, not always,
            # because if keep_alive is True we will break out of the
            # above loop after each round-trip data exchange, but we
            # will *not* want to close the channel
            self.close()
            raise
    
    def run_wait_loop(self, obj, attrname):
        try:
            while not getattr(obj, attrname):
                while self.readable():
                    self.handle_read()
        except:
            self.close()
            raise



#!/usr/bin/env python3
"""
Module ASYNC -- Asynchronous I/O Utilities
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the base class for asynchronous I/O
functionality. The idea is the same as the asyncore module
in the Python standard library, but the core is abstracted
out so it can be used with various I/O types, not just
network sockets.

There is also a modified asynchronous loop function that
allows a callback on every iteration, which is useful if a
separate event loop needs to be kept running in parallel
with the async loop (for example, a GUI event loop).
"""

import asyncore
import select
from errno import EBADF

_use_poll = True


# Need an accessor since this module gets imported into the
# package using from import *

def use_poll(flag):
    global _use_poll
    _use_poll = flag


# Monkeypatch asyncore to change the implementation of
# the read/write functions so that we pass all exceptions
# through handle_error (which re-raises them by default
# after closing the channel, so there's no need to give
# ExitNow any special treatment), and to enforce consistent
# semantics (since the semantics of asyncore's underlying
# polling loop functions have changed several times through
# Python versions, not always in ways beneficial to our
# approach here.)

# Note that for our code to work with code that uses asyncore
# but not our I/O classes, we have to check the type of each
# async object and only run our version if it's one of ours


_read = asyncore.read
_write = asyncore.write
_exc = asyncore._exception
_readwrite = asyncore.readwrite


def read(obj):
    if isinstance(obj, AsyncBase):
        try:
            obj.handle_read_event()
        except:
            obj.handle_error()
    else:
        _read(obj)


def write(obj):
    if isinstance(obj, AsyncBase):
        try:
            obj.handle_write_event()
        except:
            obj.handle_error()
    else:
        _write(obj)


def _exception(obj):
    if isinstance(obj, AsyncBase):
        try:
            obj.handle_expt_event()
        except:
            obj.handle_error()
    else:
        _exc(obj)


def readwrite(obj, flags):
    if isinstance(obj, AsyncBase):
        try:
            if flags & (select.POLLIN | select.POLLPRI):
                obj.handle_read_event()
            if flags & select.POLLOUT:
                obj.handle_write_event()
            if flags & (select.POLLERR | select.POLLHUP | select.POLLNVAL):
                obj.handle_expt_event()
        except:
            obj.handle_error()
    else:
        _readwrite(obj, flags)


asyncore.read = read
asyncore.write = write
asyncore._exception = _exception
asyncore.readwrite = readwrite


def loop(timeout=30.0, use_poll=False, map=None, count=None, callback=None):
    """Basic asynchronous polling loop.
    
    This async looping function allows a callback function
    to run on each loop iteration; if the callback returns
    ``False`` (the specific object, not just a value that tests
    false), it breaks out of the loop.
    """
    
    if map is None:
        map = asyncore.socket_map
    
    if use_poll and hasattr(select, 'poll'):
        poll_fun = asyncore.poll2
    else:
        # Need to compensate for the fact that if select breaks
        # on a bad file descriptor, it can't tell us *which* one
        # it was, so we have tell *each* object to check to see
        # if there might be an error (poll just sets the POLLHUP
        # flag for that specific fd so this hack isn't needed)
        def poll_fun(timeout, map):
            try:
                asyncore.poll(timeout, map)
            except select.error as err:
                if err.errno != EBADF:
                    raise
                else:
                    for fd, obj in map.items():
                        _exception(obj)
    
    if count is None:
        if callback is None:
            while map:
                poll_fun(timeout, map)
        else:
            while map and (callback() is not False):
                poll_fun(timeout, map)
    
    else:
        if callback is None:
            while map and count > 0:
                poll_fun(timeout, map)
                count = count - 1
        else:
            while map and (count > 0) and (callback() is not False):
                poll_fun(timeout, map)
                count = count - 1


# Monkeypatch asyncore to use enhanced loop function
asyncore.loop = loop


class AsyncBase(object):
    """Base class for async communication channel.
    
    Base class that abstracts out the core functionality
    for asynchronous I/O. Can be used to wrap any object
    that has a Unix file descriptor; the ``set_fileobj``
    method must be called with the object to be wrapped.
    Note that this class does *not* set file descriptors
    to non-blocking mode; that can't be reliably done
    here because there are too many different types of
    descriptors. Thus, users of this class must first set
    their file descriptors to non-blocking mode before
    calling the ``set_fileobj`` method.
    
    This class also allows a callback function on
    each iteration of the polling loop. This allows other
    processing to be done while waiting for I/O (one
    common use case would be keeping a GUI event loop
    running concurrently with the network polling loop).
    Note that if a callback is used this way, the
    ``poll_timeout`` field should be set to a reasonably
    short float value (e.g., 1.0--the value is in seconds).
    The default for this field is ``None`` since we use
    the self-pipe trick for signal notification, so there
    is normally no need to time out.
    """
    
    debug = False
    poll_timeout = None
    
    def __init__(self, map=None):
        if map is None:
            self._map = asyncore.socket_map
        else:
            self._map = map
        self._fileno = None
    
    def __repr__(self):
        status = [self.__class__.__module__ + "." + self.__class__.__name__]
        self.repr_status(status)
        return '<{} at {:x}>'.format((' '.join(status), id(self)))
    
    def repr_status(self, status):
        pass  # derived classes can add more stuff to repr here
    
    def fileno(self):
        return self._fileno  # so we look file-like
    
    def set_fileobj(self, obj, map=None):
        self._fileno = obj.fileno()
        self.add_channel(map)
    
    def add_channel(self, map=None):
        if map is None:
            map = self._map
        #if self._fileno in map:
        #    raise RuntimeError("{!r} tried to add itself to the map twice!".format(self))
        map[self._fileno] = self
    
    def del_channel(self, map=None):
        fd = self._fileno
        if fd is not None:
            if map is None:
                map = self._map
            if fd in map:
                del map[fd]
            self._fileno = None
    
    def close(self):
        self.del_channel()
        self.handle_close()
    
    def readable(self):
        return True
    
    def writable(self):
        return True
    
    def handle_read_event(self):
        self.handle_read()
    
    def handle_write_event(self):
        self.handle_write()
    
    def handle_expt_event(self):
        self.handle_expt()
    
    def handle_error(self):
        """More Pythonic way of handling errors.
        """
        self.close()
        raise
    
    def handle_read(self):
        """Derived classes must implement this method to read data.
        """
        raise NotImplementedError
    
    def handle_write(self):
        """Derived classes must implement this method to write data.
        """
        raise NotImplementedError
    
    def handle_expt(self):
        """Check to see if we have an error.
        
        Called when select/poll indicates that there might be an
        error on our file descriptor.
        """
        self.check_error()
    
    def check_error(self):
        """Placeholder for derived classes.
        """
        pass
    
    def handle_close(self):
        self.on_close()
    
    def on_close(self):
        """Placeholder for derived classes.
        """
        pass
    
    def do_loop(self, callback=None):
        """Basic async polling loop.
        
        Convenience looping method that allows a callback function
        to be called on each iteration of the polling loop. Note that
        we allow the callback to break us out of the loop by returning
        ``False`` (not just any false value, but the specific object
        ``False``).
        
        Note that when there are multiple async channels (e.g., in a
        socket server handling multiple connections), only one object
        should call this function (since only one async loop can run
        at a time); normally that will be the "server" object (each
        request handler then just gets added to the map for the server's
        polling loop).
        """
        loop(timeout=self.poll_timeout, use_poll=_use_poll, map=self._map, callback=callback)



#!/usr/bin/env python3
"""
Module BaseServer
Sub-Package IO.SERIAL of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the serial BaseServer class.
"""


class BaseServer(object):
    """Base class for serial I/O servers.
    
    Assumes that a subclass of ``ServerCommunicator``
    is earlier in the MRO, to provide the
    ``run_loop`` method. A serial server
    directly talks to one client at a time, so it
    just has to run the comm loop to do so. The only
    other functionality here is to ensure that the
    channel is closed when the loop exits.
    """
    
    def serve_forever(self, callback=None):
        try:
            self.run_loop(callback)
        finally:
            self.close()



#!/usr/bin/env python3
"""
SETUP-EXAMPLES script for Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script sets up symlinks in $PREFIX/bin to each of the
example programs that come with PLIB3, and byte-compiles
the examples, on systems that support symlinks (i.e.,
POSIX systems only). On other systems (i.e., Windows),
the example programs and their associated files are instead
simply copied to the Python/Scripts directory.
"""

from plib.stdlib.postinstall import setup_examples
setup_examples('plib3.io', 'plib.io')



#!/usr/bin/env python3
"""
Module ForkingServer
Sub-Package IO.BLOCKING of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``ForkingServer`` class, which
is a forking version of the blocking I/O socket server.
See the module docstring for the base class, ``SocketServer``,
for more information.
"""

import signal

from plib.stdlib.comm._processwrapper import ProcessWrapper

from plib.io.blocking import SocketServer
from plib.io.mixins import ChildWrapperMixin


class ForkingServer(ChildWrapperMixin, SocketServer):
    """Forking TCP server.
    
    This server forks a new child process for each request
    and instantiates a request handler in the child. Note
    that this server does no tracking of child processes
    once they are forked; on Unix-type systems, it sets its
    SIGCHLD handler to SIG_IGN so that the kernel will reap
    its children once they exit. If actual tracking of the
    child processes is desired, the ``SigChldServerMixin``
    class can be used to add that functionality.
    """
    
    wrapper_class = ProcessWrapper
    
    ignore_sigchld = True
    
    def server_start(self):
        super(ForkingServer, self).server_start()
        # This tells the kernel to reap our zombies for us
        # (note that we include a flag here to allow a derived
        # class, such as one using SigChldServerMixin, to
        # disable this, usually because the derived class
        # sets its own SIGCHLD handler)
        if self.ignore_sigchld and hasattr(signal, 'SIGCHLD'):
            signal.signal(signal.SIGCHLD, signal.SIG_IGN)
    
    def start_child(self, handler, conn, addr):
        super(ForkingServer, self).start_child(handler, conn, addr)
        conn.close()  # only the child needs it now



#!/usr/bin/env python3
"""
Module PersistentSocketMixin
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous PersistentSocketMixin class;
this class is factored out from PersistentSocketClient so that alternate
read/write handling can be mixed in.
"""

from plib.io.socket import ConnectMixin
from plib.io.async import AsyncConnectMixin, PersistentMixin


class PersistentSocketMixin(AsyncConnectMixin, ConnectMixin, PersistentMixin):
    """Mixin class for persistent, full-duplex asynchronous socket I/O.
    
    Can be used for both clients and servers, but intended
    mainly for clients that need connect functionality. (For
    server-side persistent sockets, you should normally use the
    ``PersistentRequestHandler`` class with ``SocketServer``.)
    """
    pass



#!/usr/bin/env python3
"""
Module SerialServerMixin
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous SerialServerMixin class.
"""

from plib.io.serial import BaseServer
from plib.io.async import ServerMixin


class SerialServerMixin(BaseServer, ServerMixin):
    """Asynchronous server-side serial I/O mixin class.
    """
    pass



#!/usr/bin/python3 -u
"""
Setup script for PLIB3.IO package
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.io import __version__ as version

name = "plib3.io"
description = "Flexible handling of I/O channels."

author = "Peter A. Donis"
author_email = "peterdonis@alum.mit.edu"

startline = 5
endspec = "The Zen of PLIB3"

dev_status = "Beta"

license = "GPLv2"

data_dirs = ["examples"]

classifiers = """
Environment :: Console
Environment :: MacOS X
Environment :: Win32 (MS Windows)
Intended Audience :: Developers
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: POSIX :: Linux
Topic :: Software Development :: Libraries :: Python Modules
"""

requires = """
plib3.stdlib (>=0.9.3)
"""

post_install = ["plib3-setup-{}.py".format(s) for s in ("io-examples",)]


if __name__ == '__main__':
    import sys
    import os
    from distutils.core import setup
    from setuputils import setup_vars
    setup(**setup_vars(globals()))
    if "install" in sys.argv:
        for scriptname in post_install:
            os.system(scriptname)



#!/usr/bin/env python3
"""
Module SocketDispatcher -- Asynchronous Socket I/O
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains a basic asynchronous socket I/O
dispatcher.
"""

import socket
from errno import EALREADY, EBADF, EINPROGRESS, EWOULDBLOCK, errorcode

from plib.io.socket import SocketIOBase

from ._async import AsyncBase

connect_in_progress_values = [EINPROGRESS, EALREADY, EWOULDBLOCK]


class SocketDispatcher(SocketIOBase, AsyncBase):
    """Base "dispatcher" class for async sockets.
    
    This dispatcher class fixes a number of minor issues
    with asyncore.dispatcher. Key changes:
    
    - Correctly handles the case where a non-blocking connect
      attempt fails; asyncore.dispatcher ends up bailing to
      ``handle_error`` on the first read or write attempt to the
      socket, but aside from being ugly, this doesn't work if
      the dispatcher won't return ``True`` for ``readable`` or
      ``writable`` until it knows the connect has succeeded--it
      will just hang forever in the polling loop--but this class
      spots the socket error and raises an exception so the loop
      exits.
    
    - The ``handle_error`` method is changed to close the socket
      and then re-raise whatever exception caused it to be called
      (much more Pythonic) -- this behavior is inherited from
      ``AsyncBase``.
    
    - The ``handle_close`` method is called from ``close``,
      instead of the reverse (having the method that's intended
      to be a placeholder call a method that needs to always be
      called makes no sense, and the naming is more consistent if
      the "handle" method is the placeholder).
    """
    
    connect_pending = False
    
    def __init__(self, sock=None, map=None):
        AsyncBase.__init__(self, map)
        SocketIOBase.__init__(self, sock)
    
    def repr_status(self, status):
        if self.accepting and self.addr:
            status.append('listening')
        elif self.connected:
            status.append('connected')
        if self.addr is not None:
            try:
                status.append('{}:{:d}'.format(*self.addr))
            except TypeError:
                status.append(repr(self.addr))
    
    def set_socket(self, sock):
        SocketIOBase.set_socket(self, sock)
        sock.setblocking(0)
        self.set_fileobj(sock, self._map)
    
    def accept(self):
        try:
            return self.socket.accept()
        except socket.error as e:
            if e.errno == EWOULDBLOCK:
                return (None, None)
            else:
                raise
    
    def connect_in_progress(self, value):
        return value in connect_in_progress_values
    
    def connect(self, address):
        self.connected = False
        self.connect_pending = False
        err = self.socket.connect_ex(address)
        # FIXME: Add Winsock return values?
        if self.connect_in_progress(err):
            # The connect will be completed asynchronously, so
            # set a flag to signal that we're waiting
            self.connect_pending = True
            self.closed = False
        elif self.connect_confirmed(err):
            self.addr = address
            self.connected = True
            self.closed = False
            self.handle_connect()
        else:
            raise socket.error(err, errorcode[err])
    
    def check_error(self):
        try:
            err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        except socket.error as e:
            # This catches EBADF, which signals that our socket has closed;
            # we check for that here because there are conditions where we
            # can't be sure we will get notified any other way (e.g., if
            # we are using select instead of poll, since select can't report
            # exceptions to a specific fd
            err = e.errno
        if err:
            self.close()
            if err != EBADF:
                # Re-raise the error so any pending operation aborts
                raise socket.error(err, errorcode[err])
    
    def check_connect(self):
        if self.connect_pending:
            self.connect_pending = False
            # We're waiting for a connect to be completed
            # asynchronously, so we need to see if it really
            # was completed or if an error occurred
            self.check_error()
            # If we get here, the connect was successful, so
            # fill in the address
            self.addr = self.socket.getpeername()
        
        # Always set this flag since we only get called if
        # it wasn't already set
        self.connected = True
    
    def readable(self):
        """Check for socket error before allowing read.
        """
        self.check_error()
        return not self.closed
    
    def writable(self):
        """Check for socket error before allowing write.
        """
        self.check_error()
        return not self.closed
    
    def keep_open_on_exc(self, e):
        # This will keep the socket open if send/recv raises EWOULDBLOCK
        # (which will happen if the fd goes unready between the return
        # of the select/poll call and the send/recv)
        return e.errno == EWOULDBLOCK
    
    def handle_read_event(self):
        if self.accepting:
            # Handle the accept--this should be the only read
            # we ever see, since we hand off the actual connection
            # to another socket
            self.handle_accept()
        else:
            # Getting a read implies that we are connected, so
            # we first check to see if we were waiting for a connect
            # to be completed asynchronously and verify it if so
            if not self.connected:
                self.check_connect()
                self.handle_connect()
            self.handle_read()
    
    def handle_write_event(self):
        if self.accepting:
            # Should never get a write event, but if we do, throw
            # it away
            return
        else:
            # Getting a write implies that we are connected, so
            # same logic as for handle_read_event above
            if not self.connected:
                self.check_connect()
                self.handle_connect()
            self.handle_write()
    
    def close(self):
        # This is the method that should be called from all the
        # places that call handle_close in asyncore.dispatcher
        self.del_channel()
        if not self.closed:
            self.handle_close()
            self.connect_pending = False
        SocketIOBase.close(self)
    
    def handle_accept(self):
        """Should only be implemented in servers.
        """
        raise NotImplementedError
    
    def handle_connect(self):
        self.on_connect()
    
    def on_connect(self):
        """Placeholder for derived classes.
        """
        pass



#!/usr/bin/env python3
"""
Module SerialPseudoSocket
Sub-Package IO.CLASSES of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the SerialPseudoSocket class. This
is a specialized serial device class that provides a
(minimal) socket-like interface, so that it can be used
in place of a socket for applications that expect one.
An example of this usage is the SerialTelnet class in
this sub-package, which uses this class as a simple
"drop-in" replacement for the Python standard library
``socket`` class to implement a telnet client that can
communicate over a serial port.
"""

from plib.io.serial import Serial


class SerialPseudoSocket(Serial):
    """A wrapper to make a serial port look minimally like a socket.
    
    Make a serial port look like a socket (at least enough so
    to fool most minimal socket-using applications).
    """
    
    def recv(self, bufsize):
        return self.read(bufsize)
    
    def send(self, data):
        return self.write(data)
    
    def sendall(self, data):
        while data:
            sent = self.write(data)
            data = data[sent:]



#!/usr/bin/env python3
"""
Module SocketClientMixin
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous SocketClientMixin class;
this class is factored out from SocketClient so that alternate
read/write handling can be mixed in.
"""

from plib.io.socket import BaseClient
from plib.io.async import AsyncConnectMixin, ClientMixin


class SocketClientMixin(AsyncConnectMixin, BaseClient, ClientMixin):
    """Asynchronous socket client mixin class.
    """
    pass



#!/usr/bin/env python3
"""
Module SigIntServerMixin
Sub-Package IO.MIXINS of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``SigIntServerMixin`` class. This
class customizes ``SigIntMixin`` for use with PLIB3 servers.
It is useful when simple termination signal functionality is
desired without all the extra frills of ``PServerBase``.
"""

from plib.stdlib.classes import SigIntMixin


class SigIntServerMixin(SigIntMixin):
    """Mixin class for PLIB3 servers to do controlled shutdown on Ctrl-C.
    
    Overrides ``server_start`` to set up the signal handler.
    
    The default of exiting on Ctrl-C (SIGINT) can be changed by
    overriding the ``term_sigs`` class field; it should contain
    a list of signals to be treated as "terminate" signals. See
    the ``PServerBase`` class for an example.
    """
    
    terminate_flag = False
    
    def server_start(self):
        # Override to set up the signal handler
        super(SigIntServerMixin, self).server_start()
        pipe = getattr(self, 'pipe', None)
        if (pipe is not None):
            # If we are using the self-pipe trick (if so, the
            # SelfPipeServerMixin class should have set up the
            # pipe in the super call above), we use its
            # mechanism to track signals; when the self-pipe
            # is read from, it will call self.signal_callback
            for sig in self.term_sigs:
                self.pipe.track_signal(sig)
        else:
            # This uses the mechanism inherited from SigIntMixin,
            # which calls self.terminate_process
            self.setup_term_sig_handler()
    
    def signal_callback(self, sig):
        # This is called if the self-pipe trick is being used, once
        # for each signal received; we only handle our signals here
        # and pass the rest on
        if sig in self.term_sigs:
            self.terminate_process()
        else:
            super(SigIntServerMixin, self).signal_callback(sig)
    
    def terminate_process(self):
        """Tell the server to break out of its ``serve_forever`` loop.
        """
        self.terminate_flag = True
    
    def keep_running(self):
        return (super(SigIntServerMixin, self).keep_running()
                and not self.terminate_flag)



#!/usr/bin/env python3
"""
TEST.IO.TEST_IO.PY -- test script for sub-package IO of package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains basic unit tests for the I/O modules in the
PLIB3.IO sub-package.
"""

import unittest
import select

from plib.io import async

from .testlib import ClientServerTest, SmallBufferTest
from .testlib_blocking import *

# Only run async tests here if poll is available

if hasattr(select, 'poll'):
    
    async.use_poll(True)  # ensure we use poll (even if multiple test modules
                          # are being run that use async)
    
    from .testlib_async import *
    
    
    class NonBlockingSocketTest(AsyncTestMixin,
                                ClientServerTest,
                                unittest.TestCase):
        pass
    
    
    class NonBlockingSocketTestMultipleTrips(NonBlockingSocketTest):
        
        number_of_trips = 3
    
    
    class NonBlockingSocketTestLargeMessage1(NonBlockingSocketTest):
        
        test_data = b"a" * 6000
    
    
    class NonBlockingSocketTestLargeMessage2(NonBlockingSocketTest):
        
        test_data = b"a" * 10000
    
    
    class NonBlockingSocketTestBufsize(AsyncShutdownTestMixin,
                                       ClientServerTest,
                                       unittest.TestCase):
        
        test_data = b"x" * AsyncShutdownClient.bufsize
    
    
    class ReadWriteTest(AsyncReadWriteTestMixin,
                        ClientServerTest,
                        unittest.TestCase):
        pass
    
    
    class ReadWriteTestMultipleTrips(ReadWriteTest):
        
        number_of_trips = 3
    
    
    class ReadWriteTestLargeMessage1(ReadWriteTest):
        
        test_data = b"a" * 6000
    
    
    class ReadWriteTestLargeMessage2(ReadWriteTest):
        
        test_data = b"a" * 10000
    
    
    class ReadWriteTestBufsize(ReadWriteTest):
        
        # total data including read/write encoding should be exactly one buffer
        test_data = b"x" * (
            AsyncReadWriteClient.bufsize
            - len(str(AsyncReadWriteClient.bufsize))
            - len(AsyncReadWriteClient.bufsep)
        )
    
    
    class ReadWriteTestSmallBuffer(AsyncReadWriteTestMixin,
                                   SmallBufferTest,
                                   unittest.TestCase):
        pass
    
    
    class TerminatorTest(AsyncTerminatorTestMixin,
                         ClientServerTest,
                         unittest.TestCase):
        pass
    
    
    class TerminatorTestMultipleTrips(TerminatorTest):
        
        number_of_trips = 3
    
    
    class TerminatorTestLargeMessage1(TerminatorTest):
        
        test_data = b"a" * 6000
    
    
    class TerminatorTestLargeMessage2(TerminatorTest):
        
        test_data = b"a" * 10000
    
    
    class TerminatorTestBufsize(TerminatorTest):
        
        # total data including terminator should be exactly one buffer
        test_data = b"x" * (
            AsyncTerminatorClient.bufsize
            - len(AsyncTerminatorClient.terminator)
        )
    
    
    class TerminatorTestSmallBuffer(AsyncTerminatorTestMixin,
                                    SmallBufferTest,
                                    unittest.TestCase):
        pass


# Blocking tests run regardless of whether poll is available

class BlockingSocketTest(BlockingTestMixin,
                         ClientServerTest,
                         unittest.TestCase):
    pass


class BlockingSocketTestMultipleTrips(BlockingSocketTest):
    
    number_of_trips = 3


class BlockingSocketTestLargeMessage1(BlockingSocketTest):
    
    test_data = b"a" * 6000


class BlockingSocketTestLargeMessage2(BlockingSocketTest):
    
    test_data = b"a" * 10000


class BlockingSocketTestBufsize(BlockingShutdownTestMixin,
                                ClientServerTest,
                                unittest.TestCase):
    
    test_data = b"x" * BlockingShutdownClient.bufsize


class BlockingSocketTestReadWrite(BlockingReadWriteTestMixin,
                                  ClientServerTest,
                                  unittest.TestCase):
    pass


class BlockingSocketTestReadWriteMultipleTrips(BlockingSocketTestReadWrite):
    
    number_of_trips = 3


class BlockingSocketTestTerminator(BlockingTerminatorTestMixin,
                                   ClientServerTest,
                                   unittest.TestCase):
    pass


class BlockingSocketTestTerminatorMultipleTrips(BlockingSocketTestTerminator):
    
    number_of_trips = 3


if __name__ == '__main__':
    # Only print the message here so it won't print multiple times
    # if the test module gets re-imported (e.g., when running under
    # Windows in the forked server process); we do this similarly
    # in the other I/O test modules that print messages
    if not hasattr(select, 'poll'):
        print("Poll not available, skipping async I/O tests for poll.")
    unittest.main()



#!/usr/bin/env python3
"""
TEST.IO.TESTLIB.PY -- utility module for PLIB3 I/O tests
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common code for the tests of the I/O
modules in PLIB3.IO.
"""

import sys
import socket
from errno import EPIPE, ECONNRESET
from functools import partial

from plib.stdlib.coll import fifo
from plib.stdlib.comm._processwrapper import ProcessWrapper
from plib.stdlib.comm._serverproxy import ServerProxy
from plib.stdlib.comm._socketwrapper import socketpair_wrapper

# FIXME: make this work on Windows
#if sys.platform == 'win32':
#    # Use threads for multi-client concurrent request tests
#    # (child processes are way too slow on Windows)
#    from plib.stdlib.comm._threadwrapper import ThreadWrapper


# Helper function to read all available data from a socket
# (note that this function assumes that the other end will
# close its socket when it is finished sending data--i.e.,
# this function can only be used as the last data transfer
# on a socket)

def recv_all(sock, bufsize=4096):
    result = b""
    while 1:
        s = sock.recv(bufsize)
        if not s:
            break
        result += s
    return result


# Server mixin class (currently just a placeholder in case we
# want to add common server functionality in future)

class IOServerMixin(object):
    pass


# Run the server listening on an ephemeral port, and send the port
# number back to the test case through a socket pair (using an
# ephemeral port ensures no port collisions between successive
# tests); we use ServerProxy so (<module_name>, <class_name>) tuples
# can be used for the server and handler classes

def run_io_server(server_class, handler_class, sock):
    proxy = ServerProxy(server_class, ('localhost', 0), handler_class)
    proxy.start_server()
    iface, port = proxy.server.socket.getsockname()
    sock.sendall(str(port).encode())
    sock.close()
    proxy.run_server()


# Fork a server and get its port number back through a socket pair
# (yes, we could use a pipe on Unix-type systems, since the data
# flow is one-way, but we don't want to have to special-case this
# code for different platforms, and we have a socketpair emulation
# already implemented for Windows)

def fork_server_with_port(server_class, handler_class):
    sock, server = socketpair_wrapper(
        partial(run_io_server, server_class, handler_class),
        ProcessWrapper)
    portstr = recv_all(sock)
    sock.close()
    return server, int(portstr)


# Base class for all I/O channel tests, contains common
# code to fork and manage the I/O server.

class IOChannelTest(object):
    
    handler_class = None
    server_class = None
    
    server = None
    server_port = None
    
    def setUp(self):
        # We allow for fork issues that are unrelated to our tests;
        # but three failures is enough to assume that there is a
        # genuine problem with the server class
        tries = 0
        while self.server is None:
            try:
                self.server, self.server_port = fork_server_with_port(
                    self.server_class,
                    self.handler_class)
            except IOError as e:
                tries += 1
                msg = str(e)
                if (msg != "fork_wait aborted!") or (tries >= 3):
                    raise
    
    def tearDown(self):
        # We check the server field so that we won't try to kill
        # the server if it's already been closed by a test
        if self.server is not None:
            self.server.end()
            self.server = None


# Client that keeps track of multiple round-trip data exchanges
# and flags early closing of the I/O channel; note that the same
# mixin class works for all I/O types (and similarly for request
# handlers and servers)

class IOClientMixin(object):
    
    result = b""
    trips_required = 1
    trips_completed = 0
    
    def trips_done(self):
        return self.trips_completed >= self.trips_required
    
    def process_data(self):
        self.trips_completed += 1
        if self.trips_done():
            # This ensures that the test won't pass until all
            # required round trips are completed
            self.result = self.read_data
            if self.keep_alive:
                # This closes the client channel after multiple trips
                # (since it won't close automatically); note that the
                # server-side channel will see the close and close
                # itself, we don't need to subclass any code there
                self.finish()
    
    def close(self):
        # First make sure we clean up
        super(IOClientMixin, self).close()
        # This flags an error if we close before the trips are
        # completed (to make sure we do indeed keep our channel
        # alive for all the trips); note that we check first to
        # make sure no other exception is being handled (if one
        # is, we don't want to clobber it)
        if not self.trips_done():
            if sys.exc_info() == (None, None, None):
                if self.keep_alive:
                    descr = "Keep-alive channel"
                else:
                    descr = "Channel"
                raise RuntimeError("{} closed early!".format(descr))


# The basic test of client/server data exchange. Note that the same
# subclassing works for client, request handler, and server classes
# of all I/O types. Also note that we can test multiple round-trip
# data exchanges by increasing the number_of_trips field in the
# test case (which fills in the client's trips_requried field).

class ClientServerTest(IOChannelTest):
    
    client_class = None
    number_of_trips = 1
    test_data = b"Python rocks!"
    
    def setUp(self):
        # Make sure we keep channels open for multiple trips
        self.client_class.trips_required = self.number_of_trips
        if self.number_of_trips > 1:
            for klass in (self.client_class, self.handler_class):
                klass.keep_alive = True
        super(ClientServerTest, self).setUp()
    
    def test_io(self):
        client = self.client_class()
        client.setup_client(('localhost', self.server_port))
        for _ in range(self.number_of_trips):
            client.client_communicate(self.test_data)
        self.assertEqual(client.result, self.test_data)


# This is to test proper behavior of the alternate read/write classes
# with small buffer sizes (meaning that the terminator or data size
# bytes may take multiple reads to arrive).

class SmallBufferTest(ClientServerTest):
    
    def setUp(self):
        for klass in (self.client_class, self.handler_class):
            klass.bufsize = 1
        super(SmallBufferTest, self).setUp()


# Here we want to test interleaved full-duplex reads and writes, so
# we have the server/request handler send back different data, except
# for the sentinel at the end.

test_done = b"All done!"
test_failed = b"Oops, missed some data!"


# We have two ways of priming the write queue, but only the first (i.e.,
# the one used when ``test_auto_push`` is False) should be used by
# actual user code (see the comments in the writable method and in
# PersistentTest below)

class PersistentTestIO(object):
    
    test_case = None
    data_queue = None
    send_list = None
    
    received_list = None
    
    def writable(self):
        # Prime the write queue if not yet primed (we do this
        # here because ``do_loop`` is only called on the client side;
        # the request handler is "run" by the server's ``do_loop``,
        # not its own, whereas this method gets called on both ends.
        # Note that after the first time through, ``send_list`` is
        # empty, so no more priming is done.
        while self.send_list:
            self.start(self.send_list.nextitem())
        # This priming method tests the ``start`` method to make
        # sure it pushes if nothing is pushed, but in the right order;
        # we do this by popping the last item in the queue and then
        # calling ``start`` with it; this should re-append it to the
        # queue and then push the first queue item; note that this
        # condition is mutually exclusive from the above (we'll have a
        # send_list or a data_queue, but never both). Note also that
        # this configuration is what I tried to implement in the actual
        # library code (in the ``PersistentMixin.writable`` method),
        # but that caused issues that don't arise when we do it here;
        # I'm not sure why there's a difference, but it's not essential
        # to figure it out (see the comments in the library method).
        if self.write_complete() and self.data_queue:
            self.start(self.data_queue.pop())
        return super(PersistentTestIO, self).writable()
    
    def start(self, data):
        if self.write_data:
            wait_count = 1
        else:
            wait_count = 0
        if self.data_queue:
            wait_count += len(self.data_queue)
        super(PersistentTestIO, self).start(data)
        if self.test_case:
            # Check to make sure we pushed data if there was anything to push
            self.test_case.assertFalse(
                self.write_complete() and self.data_queue)
            # Check to make sure we pushed in the right order
            self.test_case.assertTrue(
                (wait_count == 0) == (self.write_data == data))
    
    def process_data(self):
        if self.received_list is None:
            self.received_list = []
        self.received_list.append(self.read_data)


# Client that closes its comm loop once status is received from the
# handler (note that persistent channels have to explicitly call
# finish to break out of the loop)

class PersistentIOClientMixin(PersistentTestIO):
    
    def process_data(self):
        super(PersistentIOClientMixin, self).process_data()
        if self.read_data in (test_done, test_failed):
            self.finish()


# Request handler that tests received vs. expected data and sends back
# the status

class PersistentIOHandlerMixin(PersistentTestIO):
    
    expected_list = None
    
    def process_data(self):
        if self.read_data == test_done:
            if self.received_list == self.expected_list:
                self.start(test_done)
            else:
                self.start(test_failed)
        else:
            super(PersistentIOHandlerMixin, self).process_data()


# Server class that fills in the appropriate handler fields (we can't
# just do it on the handler class directly in the test case because
# the classes may get re-used in multiple tests; once the server is
# forked it's OK because the change won't be seen in the parent)

class PersistentIOServerMixin(IOServerMixin):
    
    client_list = None
    server_list = None
    test_auto_push = False
    
    def __init__(self, server_addr, handler_class):
        handler_class.expected_list = self.client_list
        server_sends = fifo(self.server_list)
        if self.test_auto_push:
            handler_class.data_queue = server_sends
        else:
            handler_class.send_list = server_sends
        super(PersistentIOServerMixin, self).__init__(server_addr, handler_class)


class PersistentTest(IOChannelTest):
    
    client_class = None
    
    client_list = None
    server_list = None
    
    # NOTE: Some test cases set this to True to test the ``start`` method
    # (see the comment in ``PersistentTestIO.writable`` above), but actual
    # user code should not use this hack (i.e., should not declare a
    # ``data_queue`` class member, as ``IOHandlerFactory`` and
    # ``IOClientFactory`` do above when this field is True); ``data_queue``
    # is intended to be an internal variable only
    
    test_auto_push = False
    
    # Attributes that will be temporarily copied to the server class
    # for on-passing to the handler after forking
    
    server_attrs = ('client_list', 'server_list', 'test_auto_push')
    
    def setUp(self):
        # Set up the server to fill in the handler fields after forking
        for attrname in self.server_attrs:
            oldattrname = 'old_{}'.format(attrname)
            setattr(self, oldattrname, getattr(self.server_class, attrname))
            setattr(self.server_class, attrname, getattr(self, attrname))
        super(PersistentTest, self).setUp()
    
    def tearDown(self):
        super(PersistentTest, self).tearDown()
        # Restore the server class fields to avoid clobbering other tests
        for attrname in self.server_attrs:
            oldattrname = 'old_{}'.format(attrname)
            setattr(self.server_class, attrname, getattr(self, oldattrname))
    
    def test_io(self):
        client = self.client_class()
        client.test_case = self
        client_sends = fifo(self.client_list + [test_done])
        if self.test_auto_push:
            client.data_queue = client_sends
        else:
            client.send_list = client_sends
        # FIXME: Async connects won't always raise ECONNREFUSED here, sometimes
        # it won't be received until the first read occurs inside do_loop; is
        # there a way to make it always be raised here (for consistent semantics
        # of do_connect between async and blocking)?
        client.do_connect(('localhost', self.server_port))
        # The data is already queued, so just run the comm loop
        client.do_loop()
        self.assertEqual(client.received_list, self.server_list + [test_done])


# Helper function to determine workable number of multiple/concurrent requests

def sys_request_num():
    if sys.platform == 'darwin':
        # XXX Mac OS X socket accept seems a lot slower than Linux
        return 15
    if sys.platform.startswith('freebsd'):
        # Process limit defaults seem pretty low on FreeBSD
        return 10
    if sys.platform == 'win32':
        # *Everything* is slow and clunky on Windows
        return 5
    return 100


# Test the ability of the servers to handle multiple requests; first, we just
# run each client one at a time, so the requests come in serially (but the
# shutdown of one request handler may still overlap the start of the next)

def client_test(client_class, test_data, server_port):
    client = client_class()
    try:
        client.client_communicate(test_data, ('localhost', server_port))
        return client.result
    except Exception as e:
        return str(e).encode()


class MultiRequestTest(ClientServerTest):
    
    def test_io(self):
        request_num = sys_request_num()
        results = []
        for i in range(request_num):
            results.append(client_test(self.client_class,
                           self.test_data, self.server_port))
        self.assertEqual(results, [self.test_data] * request_num)


# Now test handling of concurrent requests; we do this by forking all our
# clients so they make requests simultaneously. We try this two ways, with
# and without a "go code" sent to each client by the master test object.
# With the go code, each client process connects immediately, but doesn't
# send data until the go code is received; this means the server is forced
# to maintain as many "waiting" requests as we have clients, until they
# start receiving go codes--but on the other hand, since each client wakes
# up to send its go code sequentially, the requests will be handled and
# closed in sequence, so the server will never see more than one closing
# child process at a time. Without the go code, each client connects and
# does its round-trip data exchange as soon as it forks; here we impose a
# delay on the server end by forcing each request handler to wait for 0.5
# seconds before returning its reply, to try to keep as many concurrent
# connections pending as possible, so that requests will be handled in
# parallel (and consequently the server will be handling more than one
# closing child process at a time). Since we're not sure which case is
# harder for the server to handle, we test both ways.

go_code = b"1"


def f_go(client_class, test_data, server_port, sock):
    try:
        client = client_class()
        client.setup_client(('localhost', server_port))
        try:
            go = sock.recv(1)
            if go == go_code:
                try:
                    client.client_communicate(test_data)
                    sock.sendall(client.result)
                except Exception as e:
                    sock.sendall(str(e).encode())
        finally:
            client.close()
    finally:
        sock.close()


def f_nogo(client_class, test_data, server_port, sock):
    try:
        result = client_test(client_class, test_data, server_port)
        sock.sendall(result)
    finally:
        sock.close()


def r(fsock, fchild, use_gocode, test_data):
    result = ""
    try:
        try:
            if use_gocode:
                fsock.sendall(go_code)
            result = recv_all(fsock)
        except socket.error as e:
            if e.errno not in (EPIPE, ECONNRESET):
                raise
    finally:
        fsock.close()
    exitcode = fchild.exitcode()
    return (exitcode, result)


def multi_client_test(client_class, test_data, server_port,
                      use_gocode, wrapper_class):
    
    if use_gocode:
        f = f_go
    else:
        f = f_nogo
    fsock, fchild = socketpair_wrapper(
        partial(f, client_class, test_data, server_port),
        wrapper_class)
    return partial(r, fsock, fchild, use_gocode, test_data)


class ConcurrentRequestTest(ClientServerTest):
    
    use_gocode = False
    
    def setUp(self):
        # Make sure the server's listen queue is large enough so
        # connections won't be refused (this is probably overkill
        # as the queue shouldn't build up this high because most
        # of the connections should be accepted quickly, but we want
        # to make sure)
        self.server_class.request_queue_size = sys_request_num()
        super(ConcurrentRequestTest, self).setUp()
    
    def test_io(self):
        # FIXME: make this work on Windows
        ## Use threads on Windows since subprocesses are so slow;
        ## otherwise use child processes for each client
        #if sys.platform == 'win32':
        #    wrapper_class = ThreadWrapper
        #else:
        wrapper_class = ProcessWrapper
        request_num = sys_request_num()
        results = []
        for i in range(request_num):
            results.append(multi_client_test(
                self.client_class,
                self.test_data,
                self.server_port,
                self.use_gocode,
                wrapper_class))
        for i, result in enumerate(results):
            results[i] = result()
        self.assertEqual(results, [(0, self.test_data)] * request_num)



#!/usr/bin/env python3
"""
Module ClientMixin
Sub-Package IO.ASYNC of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous ClientMixin class.
"""

from plib.io.async import AsyncCommunicator
from plib.io.comm import ClientCommunicator


class ClientMixin(AsyncCommunicator, ClientCommunicator):
    """Mixin class for async client-side I/O channel.
    """
    
    def run_wait_loop(self, obj, attrname):
        def callback():
            if getattr(obj, attrname):
                return False
        self.do_loop(callback)



#!/usr/bin/env python3
"""
Module SerialBase
Sub-Package IO.BLOCKING of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O SerialBase class.
"""

from plib.io.serial import SerialData, SerialIOBase


class SerialBase(SerialData, SerialIOBase):
    """Base class for blocking serial I/O with data handling.
    """
    
    blocking_mode = True



#!/usr/bin/env python3
"""
Module BaseIO
Sub-Package IO of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the BaseIO class, which defines a
basic abstract interface for an I/O type. The serial
and socket I/O types subclass this base class. The
``BaseData`` class assumes that a subclass of this class
is earlier in its MRO, so that all ``BaseIO`` methods
are available with their defined semantics.
"""


class BaseIO(object):
    """Base class for I/O types.
    
    This class cannot be instantiated directly, but serves as
    a common subclass defining the interface for all I/O types.
    
    Methods::
    
    - ``dataread``: read up to ``bufsize`` bytes of data and
      return the data read as a ``str``.
    
    - ``datawrite``: write as many bytes of ``data`` (which must
      be a ``str``) as possible, and return the number of bytes
      written.
    
    - ``close``: close the I/O device; after this method is called,
      calls to any other method may produce undefined behavior.
    """
    
    def dataread(self, bufsize):
        """Read up to ``bufsize`` bytes of data, return data read.
        """
        raise NotImplementedError
    
    def datawrite(self, data):
        """Try to write data, return number of bytes written.
        """
        raise NotImplementedError
    
    def close(self):
        """Close the I/O device.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
CHAT_SERVER.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Python implementation of a "chat" server, which can have
multiple "clients", each of which sees messages sent by all
the others. The chat client and server both demonstrate
simple but illustrative use cases for the asynchronous
"persistent" I/O classes in ``plib.io.async``.
Key aspects of the server:

- The "asynchronous" part: this server has to maintain
  common state between multiple request handlers, since each
  handler has to pass on its messages to all the others.
  This is simple with an async server, as you can see from
  the code below. A forking server would have to somehow
  implement message passing between handlers and the server,
  which would require, at the least, extra sockets (or pipes,
  or some other IPC mechanism), and would impose a lot of overhead
  on what is essentially a simple thing. Even a threading
  server, which could provide shared state between handlers,
  would have to protect that state with mutexes so multiple
  handlers could not mutate it at the same time.

- The "persistent" part: each handler has to interleave reading
  messages from its client, and passing them on, with sending
  messages from all the other handlers to its client. Since
  these things can happen at arbitrary times, the handler can't
  make any assumptions about which type of I/O will happen in
  what order, as a normal request handler would. This is the
  type of use case that the "persistent" I/O classes in this
  library are meant for.
"""

import sys
import os
import signal

from plib.io.mixins import SigIntServerMixin
from plib.io.async import (
    PersistentRequestHandlerWithTerminator, SocketServer)


class ChatRequestHandler(PersistentRequestHandlerWithTerminator):
    """Test chat request handler.
    
    Writes initial greeting message before starting
    round-trip data exchange.
    
    Echoes data received to stdout and to all other clients.
    """
    
    terminator = os.linesep.encode()  # so tools like netcat can talk to us
    
    handlers = []  # easy hack to avoid a global variable
    
    def on_connect(self):
        sys.stdout.write("Chat server accepted connection.{}".format(
                         os.linesep))
        # TerminatorReadWrite will automatically add
        # the line break when sending to client
        self.start(b"Connected to chat server.")
        # We'll now start getting chat lines from other clients
        self.handlers.append(self)
    
    def process_data(self):
        if self.read_data:
            # Add back line break when writing to stdout
            # since TerminatorReadWrite strips it
            sys.stdout.write("{}{}".format(self.read_data.decode(), os.linesep))
        for handler in self.handlers:
            if handler is not self:
                handler.start(self.read_data)
    
    def on_close(self):
        self.handlers.remove(self)
        sys.stdout.write("Chat server connection closed.{}".format(
                         os.linesep))


class ChatServer(SigIntServerMixin, SocketServer):
    """Test chat server.
    
    Adds some status messages to stdout for testing.
    
    Exits on Ctrl-C, neatly (i.e., without the messy traceback
    that Python would print if we used the default of letting
    Ctrl-C raise a KeyboardInterrupt).
    """
    
    allow_reuse_address = True
    term_sigs = SigIntServerMixin.term_sigs + [signal.SIGTERM]
    
    def serve_forever(self):
        sys.stdout.write("Chat server listening.{}".format(
                         os.linesep))
        super(ChatServer, self).serve_forever()
    
    def server_close(self):
        super(ChatServer, self).server_close()
        sys.stdout.write("Chat server exiting.{}".format(
                         os.linesep))


if __name__ == '__main__':
    ChatServer(("localhost", 9999), ChatRequestHandler).serve_forever()



#!/usr/bin/env python3
"""
TEST.IO.TEST_PERSISTENT_NOPOLL.PY -- test script for sub-package IO of package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the persistent async I/O
classes in the PLIB3.IO sub-package, with the poll function
disabled (so that select must be used).
"""

import unittest

from plib.io import async

from .testlib import PersistentTest
from .testlib_persistent import ReadWriteTestMixin, TerminatorTestMixin

async.use_poll(False)  # force use of select in all async loops


class XPersistentTestNoPoll(ReadWriteTestMixin, PersistentTest, unittest.TestCase):
    
    client_list = [b"Python rocks!", b"Try it today!", b"You'll be glad you did!"]
    server_list = [b"You betcha!", b"It's *much* better than Perl!", b"And don't even *mention* C++!"]


class XPersistentTestNoPollWithPush(XPersistentTestNoPoll):
    
    test_auto_push = True


class XPersistentTestLargeMessage1NoPoll(XPersistentTestNoPoll):
    
    client_list = [b"a" * 6000, b"b" * 6000, b"c" * 6000]
    server_list = [b"1" * 6000, b"2" * 6000, b"3" * 6000]


class XPersistentTestLargeMessage2NoPoll(XPersistentTestNoPoll):
    
    client_list = [b"a" * 10000, b"b" * 10000, b"c" * 10000]
    server_list = [b"1" * 10000, b"2" * 10000, b"3" * 10000]


class YPersistentTestNoPoll(TerminatorTestMixin, PersistentTest, unittest.TestCase):
    
    client_list = [b"Python rocks!", b"Try it today!", b"You'll be glad you did!"]
    server_list = [b"You betcha!", b"It's *much* better than Perl!", b"And don't even *mention* C++!"]


class YPersistentTestNoPollWithPush(YPersistentTestNoPoll):
    
    test_auto_push = True


class YPersistentTestLargeMessage1NoPoll(YPersistentTestNoPoll):
    
    client_list = [b"a" * 6000, b"b" * 6000, b"c" * 6000]
    server_list = [b"1" * 6000, b"2" * 6000, b"3" * 6000]


class YPersistentTestLargeMessage2NoPoll(YPersistentTestNoPoll):
    
    client_list = [b"a" * 10000, b"b" * 10000, b"c" * 10000]
    server_list = [b"1" * 10000, b"2" * 10000, b"3" * 10000]


if __name__ == '__main__':
    unittest.main()
